#ifndef CSPECTRUM_H
#define CSPECTRUM_H

#include <vector>
#include "corbifold.h"

using std::vector;

class CSpectrum {
public:
// member functions
  CSpectrum();
  CSpectrum(const SConfig &Vacuum, SUSYMultiplet Multiplet);
  ~CSpectrum();

  bool                        operator==(const CSpectrum &Spectrum2) const;
  bool                        GaugeGroupFromBothE8Factors() const;
  unsigned                    CompareE8Factors() const;
  bool                        InterchangeE8xE8(CSpectrum &result) const;
  bool                        IsSpectrumEmpty() const;
  void                        AddIdentifier(const double &NewIdentifier) {this->AdditionalIdentifier.push_back(NewIdentifier);};
  
  const vector<double>       &GetAdditionalIdentifier() const {return this->AdditionalIdentifier;};
  const SUSYMultiplet        &GetSUSYMultiplet() const {return this->Multiplet;};
  const vector<unsigned>     &GetMultiplicity() const {return this->Multiplicity;};
  const vector<vector<int> > &GetRepresentations() const {return this->Representations;};
  const vector<vector<int> > &GetAlgebra() const {return this->Algebra;};
  const unsigned             &GetPosition_of_and_in_GaugeGroup() const {return this->Position_of_and_in_GaugeGroup;};
  
private:
// member variables
  vector<double>              AdditionalIdentifier;

  SUSYMultiplet               Multiplet;
  vector<unsigned>            Multiplicity;
  vector<vector<int> >        Representations;
  
  vector<vector<int> >        Algebra;
  unsigned                    Position_of_and_in_GaugeGroup;
  
// member functions
  bool                        ReOrder();
};

#endif
